#
# (c) 2004 Ivan 'Dark Dragon' Tivonenko
#
# Contents are licensed under the Nebula license.
#

from Plex import *
from Plex.Traditional import re

letter = Range("AZaz")
digit = Range("09")
name = letter + Rep(letter | digit)
number = Rep1(digit)
space = Any(" \t\n")

class CCScriptMetaInfoScanner(Scanner):

  out_on = 0

  def token_scriptclass(self, text):
    if self.out_on: print 'token_scriptclass :', text
    self.scriptclass = text
    self.begin('doxy_comment')

  def token_cppclass(self, text):
    if self.out_on: print 'token_cppclass :', text
    self.cppclass = text
    self.begin('doxy_comment')

  def token_superclass(self, text):
    if self.out_on: print 'token_superclass :', text
    self.superclass = text
    self.begin('doxy_comment')
  
  
  def token_classinfo(self, text):
    self.classinfo += text



  def token_cmd(self, text):
    if self.out_on: print 'token_cmd:', text
    self.__current_cmd_name = text
    self.cmds[self.__current_cmd_name] = {}
    self.cmds[self.__current_cmd_name]['input'] = []
    self.cmds[self.__current_cmd_name]['output'] = []
    self.cmds[self.__current_cmd_name]['info'] = ''
    self.begin('doxy_comment')
    
  def token_input_type(self, text):
    if self.out_on: print 'token_input_type:', text
    if text == 'v':
      self.cmds[self.__current_cmd_name]['input'].append({'name' : 'void', 'type' : 'v'})
    else:
      self.__current_cmd_arg_type = text


  def token_output_type(self, text):
    if self.out_on: print 'token_output_type:', text
    if text == 'v':
      self.cmds[self.__current_cmd_name]['output'].append({'name' : 'void', 'type' : 'v'})
    else:
      self.__current_cmd_arg_type = text

  
  def token_cmdinfo(self, text):
    self.cmds[self.__current_cmd_name]['info'] += text


  def token_arg_var_name_start_o(self, text):
    self.__prev_state = 'state_output'
    self.begin('state_arg_var_name')

  def token_arg_var_name_start_i(self, text):
    self.__prev_state = 'state_input'
    self.begin('state_arg_var_name')

  def token_arg_var_name_end(self, text):
    self.begin(self.__prev_state)
    self.__prev_state = ''


  def token_arg_var_name(self, text):
    if self.out_on: print 'token_arg_var_name:', text
    if self.__current_cmd_arg_type == '':
      raise ValueError, "var name without type!"

    direction = 'input'
    if self.__prev_state == 'state_output': direction = 'output'
    self.cmds[self.__current_cmd_name][direction].append({'name' :text, 'type' : self.__current_cmd_arg_type})
    self.__current_cmd_arg_type = ''


  def token_enum(self, text):
    if self.out_on: print 'token_enum:', text

    direction = 'input'
    if self.__prev_state == 'state_output': direction = 'output'
    cmd = self.cmds[self.__current_cmd_name][direction]
    var = cmd[len(cmd)-1]
    if 'enum' not in var: var['enum'] = []
    var['enum'].append(text)
    
  doxy_comment_begin = Str("/**")
  doxy_comment_char = AnyBut("*/")
  doxy_comment_end = Str("*/")
  scriptclass = Str("@scriptclass")
  cppclass = Str("@cppclass")
  superclass = Str("@superclass")
  classinfo = Str("@classinfo")
  cmd = Str("@cmd")
  t_input = Str("@input")
  t_output = Str("@output")
  t_info = Str("@info")

  info = AnyBut("*") + Str("/")

  string = Str('"') + (Rep(AnyBut('"')) | Str('""')) + Str('"')
  letter = Range("AZaz") | Any("_")
  name = letter + Rep(letter | digit)

  arg_var_name = Str('(') + Rep(AnyBut(')')) + Str(')')
  arg_var_name_start = Str('(') 
  arg_var_name_end   = Str(')') 
  before_at = Bol + Rep(Any(' ')) + AnyBut('@')

  lexicon = Lexicon([
    (number,        IGNORE),
    (space,         IGNORE),
    (doxy_comment_begin, Begin('doxy_comment')),
    State('doxy_comment', [
      (scriptclass, Begin('state_scriptclass')),
      (cppclass, Begin('state_cppclass')),
      (superclass, Begin('state_superclass')),
      (classinfo, Begin('state_classinfo')),
      (cmd, Begin('state_cmd')),
      (t_input, Begin('state_input')),
      (t_output, Begin('state_output')),
      (t_info, Begin('state_info')),

      
      (doxy_comment_end, Begin('')),
      (AnyChar, IGNORE)
    ]),
    State('state_scriptclass', [
      (name, token_scriptclass),
      (AnyChar, IGNORE)
    ]),
    State('state_cppclass', [
      (name, token_cppclass),
      (AnyChar, IGNORE)
    ]),
    State('state_superclass', [
      (name, token_superclass),
      (AnyChar, IGNORE)
    ]),
    State('state_classinfo', [
      (AnyChar, token_classinfo),
      (doxy_comment_end, Begin(''))
    ]),
    State('state_cmd', [
      (name, token_cmd),
      (AnyChar, IGNORE)
    ]),
    State('state_input', [
      (name, token_input_type),
#      (arg_var_name, token_input_var_name),
      (arg_var_name_start, token_arg_var_name_start_i),
      (t_output, Begin('state_output')),
      (AnyChar, IGNORE)
    ]),
    State('state_output', [
      (name, token_output_type),
#      (arg_var_name, token_output_var_name),
      (arg_var_name_start, token_arg_var_name_start_o),
      (t_info, Begin('state_info')),
      (doxy_comment_end, Begin('')),
      (AnyChar, IGNORE)
    ]),
    State('state_arg_var_name', [
#      (name, token_arg_var_name),
      (Rep(AnyBut(")=")), token_arg_var_name),
      (Str("="), Begin('state_enum')),
      (arg_var_name_end, token_arg_var_name_end),
      (AnyChar, IGNORE)
    ]),
    State('state_enum', [
      (name, token_enum),
      (arg_var_name_end, token_arg_var_name_end),
      (AnyChar, IGNORE)
    ]),

    State('state_info', [
      (AnyChar, token_cmdinfo),
      (doxy_comment_end, Begin(''))
    ]),
    (AnyChar, IGNORE)
  ])

  def __init__(self, file, name):
    Scanner.__init__(self, self.lexicon, file, name)
    self.scriptclass = ''
    self.cppclass = ''
    self.superclass = ''
    self.classinfo = ''
    self.__current_cmd_name = ''
    self.__current_cmd_arg_type = ''
    self.cmds = {}



